"use strict";

var pr = require("../../lib");
var util = require("util");

require("should");
require("source-map-support").install();

describe("Parser.repeatSeparated", function () {
  it("works", function () {
    var p = pr.repeatSeparated("hi", ",");
    var rv = p.execute("hi,hi,hi");
    rv.state.pos.should.equal(8);
    rv.value.should.eql(["hi", "hi", "hi"]);
  });

  describe("comma-separated numbers", function () {
    var p = pr.repeatSeparated(pr.regex(/\d+/).onMatch(function (x) {
      return x[0];
    }), /\s*,\s*/);

    it("matches one", function () {
      var rv = p.execute("98");
      rv.state.pos.should.equal(2);
      rv.value.should.eql(["98"]);
    });

    it("matches several", function () {
      var rv = p.execute("98, 99 ,100");
      rv.state.pos.should.equal(11);
      rv.value.should.eql(["98", "99", "100"]);
    });

    it("map", function () {
      var rv = p.onMatch(function (x) {
        return x.map(function (n) {
          return parseInt(n, 10);
        });
      }).execute("98, 99 ,100");
      rv.state.pos.should.equal(11);
      rv.value.should.eql([98, 99, 100]);
    });

    it("ignores trailing separators", function () {
      var rv = p.execute("98, wut");
      rv.state.pos.should.equal(2);
      rv.value.should.eql(["98"]);
    });
  });
});
//# sourceMappingURL=test_repeat_separated.js.map